@@ -7,12 +7,14 @@ LogRatioTDCoxLasso <- function(x,
                                wcov,
                                a=1,
                                mu=1,
+                               maxiter=100,
                                ncv=5,
                                foldid=NULL,
                                step2=FALSE,
                                progress=TRUE,
                                plot=TRUE,
-                               mcv="Deviance"){
+                               mcv="Deviance",
+                               ncore=1){
   
   ptm <- proc.time()
   
@@ -42,7 +44,7 @@ LogRatioTDCoxLasso <- function(x,
   sfun = d - expect
   # hfun <- expect - sapply(t,function(x) sum(1/denomj[which(tj <= x)]^2)) + 1e-8 # hessian
   # z <- sfun/hfun
-
+  
   if (a > 0){
     lambda0 <- max(abs(t(sfun) %*% x))/(a*n)
   }else if (a == 0){
@@ -57,7 +59,7 @@ LogRatioTDCoxLasso <- function(x,
   
   if (progress) cat("Algorithm running for full dataset: \n")
   
-  fullfit <- cox_timedep_enet_al(x,t0,t1,d,tj,length.lambda,mu,100,lambda,wcov,a,adjust,ncov,devnull,progress)
+  fullfit <- cox_timedep_enet_al(x,t0,t1,d,tj,length.lambda,mu,maxiter,lambda,wcov,a,adjust,ncov,devnull,progress)
   lidx <- which(fullfit$loglik != 0 | !is.nan(fullfit$loglik))
   
   dev <- -2*fullfit$loglik[lidx]
@@ -82,66 +84,122 @@ LogRatioTDCoxLasso <- function(x,
     
     # labels <- caret::createFolds(factor(d),k=ncv)
     
-    for (cv in 1:ncv){
+    if (ncore == 1){
       
-      if (progress) cat(paste0("Algorithm running for cv dataset ",cv," out of ",ncv,": \n"))
-      
-      # train.x <- x[-labels[[cv]],]
-      # train.d <- d[-labels[[cv]]]
-      # train.t <- t[-labels[[cv]]]
-      # test.x <- x[labels[[cv]],]
-      # test.d <- d[labels[[cv]]]
-      # test.t <- t[labels[[cv]]]
-      
-      train.x <- x[labels!=cv,]
-      train.d <- d[labels!=cv]
-      train.t0 <- t0[labels!=cv]
-      train.t1 <- t1[labels!=cv]
-      train.tt <- y[labels!=cv,1:2]
-      
-      test.x <- x[labels==cv,]
-      test.d <- d[labels==cv]
-      test.t0 <- t0[labels==cv]
-      test.t1 <- t1[labels==cv]
-      test.tt <- y[labels==cv,1:2]
-      
-      cv.devnull <- 0
-      train.tj <- sort(train.t1[train.d==1])
-      for (j in 1:length(train.tj)){
-        cv.devnull <- cv.devnull + log(sum(train.t1 >= train.tj[1] & train.t0 < train.tj[1]))
+      for (cv in 1:ncv){
+        
+        if (progress) cat(paste0("Algorithm running for cv dataset ",cv," out of ",ncv,": \n"))
+        
+        # train.x <- x[-labels[[cv]],]
+        # train.d <- d[-labels[[cv]]]
+        # train.t <- t[-labels[[cv]]]
+        # test.x <- x[labels[[cv]],]
+        # test.d <- d[labels[[cv]]]
+        # test.t <- t[labels[[cv]]]
+        
+        train.x <- x[labels!=cv,]
+        train.d <- d[labels!=cv]
+        train.t0 <- t0[labels!=cv]
+        train.t1 <- t1[labels!=cv]
+        train.tt <- y[labels!=cv,1:2]
+        
+        test.x <- x[labels==cv,]
+        test.d <- d[labels==cv]
+        test.t0 <- t0[labels==cv]
+        test.t1 <- t1[labels==cv]
+        test.tt <- y[labels==cv,1:2]
+        
+        cv.devnull <- 0
+        train.tj <- sort(train.t1[train.d==1])
+        for (j in 1:length(train.tj)){
+          cv.devnull <- cv.devnull + log(sum(train.t1 >= train.tj[1] & train.t0 < train.tj[1]))
+        }
+        cv.devnull <- 2*cv.devnull
+        
+        cvfit <- cox_timedep_enet_al(train.x,train.t0,train.t1,train.d,train.tj,length(lidx),mu,maxiter,lambda[lidx],wcov,a,adjust,ncov,cv.devnull,progress)
+        
+        cv.devnull <- 0
+        loglik <- rep(0,length(lidx))
+        linprod <- test.x %*% cvfit$beta
+        
+        if (mcv == "Deviance"){
+          
+          test.tj <- sort(test.t1[test.d==1])
+          for (j in 1:length(test.tj)){
+            cv.devnull <- cv.devnull + log(sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]))
+            if (sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]) > 1){
+              cvdev[,cv] <- cvdev[,cv] + linprod[test.t1 == test.tj[j],] - 
+                log(colSums(exp(linprod[test.t1 >= test.tj[j] & test.t0 < test.tj[j],])) + 1e-8)
+            }else if (sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]) == 1){
+              cvdev[,cv] <- cvdev[,cv] + linprod[test.t1 == test.tj[j],] - 
+                log(exp(linprod[test.t1 >= test.tj[j] & test.t0 < test.tj[j],]) + 1e-8) 
+            }
+            #- log(accu(link(widx))+1e-8)
+          }
+          
+          cvdevnull[cv] <- 2*cv.devnull
+          cvdev[,cv] <- -2*cvdev[,cv]
+          
+        }
+        
+        # cvmse[,cv] <- apply(cbind(1,test.x) %*% rbind(t(cvfit$beta0),cvfit$beta),2,function(x) sum((test.y - exp(x)/(1+exp(x)))^2)/length(test.y))
+        
       }
-      cv.devnull <- 2*cv.devnull
       
-      cvfit <- cox_timedep_enet_al(train.x,train.t0,train.t1,train.d,train.tj,length(lidx),mu,100,lambda[lidx],wcov,a,adjust,ncov,cv.devnull,progress)
+    }else if (ncore > 1){
       
-      cv.devnull <- 0
-      loglik <- rep(0,length(lidx))
-      linprod <- test.x %*% cvfit$beta
+      if (progress) warning(paste0("Using ", ncore ," core for cross-validation computation."))
       
-      if (mcv == "Deviance"){
+      cl <- makeCluster(ncore)
+      registerDoParallel(cl)
+      
+      cvdev <- foreach(cv=1:ncv,.combine=cbind) %dopar% {
+        
+        train.x <- x[labels!=cv,]
+        train.d <- d[labels!=cv]
+        train.t0 <- t0[labels!=cv]
+        train.t1 <- t1[labels!=cv]
+        train.tt <- y[labels!=cv,1:2]
+        
+        test.x <- x[labels==cv,]
+        test.d <- d[labels==cv]
+        test.t0 <- t0[labels==cv]
+        test.t1 <- t1[labels==cv]
+        test.tt <- y[labels==cv,1:2]
+        
+        cv.devnull <- 0
+        train.tj <- sort(train.t1[train.d==1])
+        for (j in 1:length(train.tj)){
+          cv.devnull <- cv.devnull + log(sum(train.t1 >= train.tj[1] & train.t0 < train.tj[1]))
+        }
+        cv.devnull <- 2*cv.devnull
+        
+        cvfit <- cox_timedep_enet_al(train.x,train.t0,train.t1,train.d,train.tj,length(lidx),mu,maxiter,lambda[lidx],wcov,a,adjust,ncov,cv.devnull,FALSE)
+        
+        cv.dev <- rep(0,length(lidx))
+        linprod <- test.x %*% cvfit$beta
         
         test.tj <- sort(test.t1[test.d==1])
         for (j in 1:length(test.tj)){
-          cv.devnull <- cv.devnull + log(sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]))
           if (sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]) > 1){
-            cvdev[,cv] <- cvdev[,cv] + linprod[test.t1 == test.tj[j],] - 
+            cv.dev <- cv.dev + linprod[test.t1 == test.tj[j],] - 
               log(colSums(exp(linprod[test.t1 >= test.tj[j] & test.t0 < test.tj[j],])) + 1e-8)
           }else if (sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]) == 1){
-            cvdev[,cv] <- cvdev[,cv] + linprod[test.t1 == test.tj[j],] - 
+            cv.dev <- cv.dev + linprod[test.t1 == test.tj[j],] - 
               log(exp(linprod[test.t1 >= test.tj[j] & test.t0 < test.tj[j],]) + 1e-8) 
           }
-          #- log(accu(link(widx))+1e-8)
         }
         
-        cvdevnull[cv] <- 2*cv.devnull
-        cvdev[,cv] <- -2*cvdev[,cv]
-        
+        cv.dev <- -2*cv.dev
+        cv.dev
       }
       
-      # cvmse[,cv] <- apply(cbind(1,test.x) %*% rbind(t(cvfit$beta0),cvfit$beta),2,function(x) sum((test.y - exp(x)/(1+exp(x)))^2)/length(test.y))
+      stopCluster(cl)
       
     }
     
+    
+    
     mean.cvdev <- rowMeans(cvdev)
     se.cvdev <- apply(cvdev,1,function(x) sd(x)/sqrt(ncv))
     
@@ -161,6 +219,8 @@ LogRatioTDCoxLasso <- function(x,
                 a=a,
                 loss=fullfit$loss[lidx],
                 mse=fullfit$mse[lidx],
+                tol=fullfit$tol[lidx],
+                iters=fullfit$iters[lidx],
                 cvdev.mean=mean.cvdev,
                 cvdev.se=se.cvdev,
                 best.beta=best.beta,
@@ -230,9 +290,11 @@ LogRatioTDCoxLasso <- function(x,
           }
           
           if (ncol(x.select.min) > 1){
-            stepglmnet <- cv.glmnet(x=x.select.min,y=Surv(t0,t1,d),type.measure = "deviance",family="cox")
+            stepglmnet <- suppressWarnings(cv.glmnet(x=x.select.min,y=Surv(t0,t1,d),type.measure = "deviance",family="cox"))
             x.select.min <- x.select.min[,which(stepglmnet$glmnet.fit$beta[,stepglmnet$index[1]]!=0)]
             idxs <- idxs[,which(stepglmnet$glmnet.fit$beta[,stepglmnet$index[1]]!=0)]
+          }else{
+            idxs <- as.vector(idxs)
           }
           
           df_step2 <- data.frame(t0=t0,t1=t1,d=d,x=x.select.min)
@@ -264,9 +326,11 @@ LogRatioTDCoxLasso <- function(x,
           }
           
           if (ncol(x.select.min) > 1){
-            stepglmnet <- cv.glmnet(x=x.select.min,y=Surv(t0,t1,d),type.measure = "deviance",family="cox")
+            stepglmnet <- suppressWarnings(cv.glmnet(x=x.select.min,y=Surv(t0,t1,d),type.measure = "deviance",family="cox"))
             x.select.min <- x.select.min[,which(stepglmnet$glmnet.fit$beta[,stepglmnet$index[1]]!=0)]
             idxs <- idxs[,which(stepglmnet$glmnet.fit$beta[,stepglmnet$index[1]]!=0)]
+          }else{
+            idxs <- as.vector(idxs)
           }
           
           df_step2 <- data.frame(t0=t0,t1=t1,d=d,x=x.select.min)
@@ -321,7 +385,7 @@ LogRatioTDCoxLasso <- function(x,
           # if(is.null(x.select.min)) break
           
           if (ncol(x.select.min) > 1){
-            stepglmnet <- cv.glmnet(x=x.select.min,y=Surv(t0,t1,d),type.measure = "deviance",family="cox")
+            stepglmnet <- suppressWarnings(cv.glmnet(x=x.select.min,y=Surv(t0,t1,d),type.measure = "deviance",family="cox"))
             
             if (length(taxidx) < 2){
               
@@ -395,7 +459,7 @@ LogRatioTDCoxLasso <- function(x,
           # if(is.null(x.select.min)) break
           
           if (ncol(x.select.min) > 1){
-            stepglmnet <- cv.glmnet(x=x.select.min,y=Surv(t0,t1,d),type.measure = "deviance",family="cox")
+            stepglmnet <- suppressWarnings(cv.glmnet(x=x.select.min,y=Surv(t0,t1,d),type.measure = "deviance",family="cox"))
             
             if (length(taxidx) < 2){
               
