@@ -3,7 +3,7 @@
 #' @description Simulate a dataset from log-ratio model.
 #' @param n An integer of sample size
 #' @param p An integer of number of features (taxa).
-#' @param model Type of models associated with outcome variable, can be "linear", "binomial", "cox", "finegray", or "timedep" (survival endpoint with time-dependent features).
+#' @param model Type of models associated with outcome variable, can be "linear", "binomial", "cox", "finegray", "gee" (scalar outcome with time-dependent features), or "timedep" (survival endpoint with time-dependent features).
 #' @param weak Number of features with \code{weak} effect size.
 #' @param strong Number of features with \code{strong} effect size.
 #' @param weaksize Actual effect size for \code{weak} effect size. Must be positive.
@@ -12,6 +12,12 @@
 #' @param rho Parameter controlling the correlated structure between taxa. Ranges between 0 and 1.
 #' @param timedep_slope If \code{model} is "timedep", this parameter specifies the slope for the feature trajectories. Please refer to the Simulation section of the manuscript for more details.
 #' @param timedep_cor If \code{model} is "timedep", this parameter specifies the sample-wise correlations between longitudinal features. Please refer to the Simulation section of the manuscript for more details.
+#' @param geetype If \code{model} is "gee", \code{geetype} is the type of GEE outcomes. Now support "gaussian" and "binomial".
+#' @param m If \code{model} is "gee", \code{m} is the number of repeated measurements per subject.
+#' @param corstr If \code{model} is "gee", \code{corstr} is the working correlation structure. Now support "independence", "exchangeable", and "AR-1".
+#' @param sdvec If \code{model} is "gee" and \code{geetype} is "gaussian", \code{sdvec} is the vector of standard deviations of each outcome variable.
+#' @param rhogee If \code{model} is "gee", \code{rhogee} is the correlation parameter between longitudinal outcomes under the selected working correlation structure.
+#' @param geeslope If \code{model} is "gee", \code{geeslope} is the linear time effect.
 #' @param longitudinal_stability If \code{model} is "timedep", this is a binary indicator which determines whether the trajectories are more stable (\code{TRUE}) or more volatile (\code{FALSE}).
 #' @param ncov Number of covariates that are not compositional features.
 #' @param betacov Coefficients corresponding to the covariates that are not compositional features.
@@ -33,6 +39,7 @@
 #' @importFrom caret createFolds
 #' @importFrom stats dist rbinom rexp rmultinom rnorm runif sd step glm binomial gaussian na.omit
 #' @importFrom msm dpexp ppexp rpexp
+#' @importFrom mvtnorm rmvnorm
 #' @useDynLib FLORAL
 #' @export
 
@@ -47,6 +54,12 @@ simu <- function(n = 100,
                  rho=0,
                  timedep_slope=NULL,
                  timedep_cor=NULL,
+                 geetype="gaussian",
+                 m=4,
+                 corstr="exchangeable",
+                 sdvec=NULL,
+                 rhogee=0.8,
+                 geeslope=2.5,
                  longitudinal_stability=TRUE,
                  ncov=0,
                  betacov=0,
@@ -82,6 +95,25 @@ simu <- function(n = 100,
     }else{
       timedep_slope <- timedep_slope
     }
+  }else if (model == "gee"){
+    
+    id.vect <- rep(1:n, each = m)
+    n0 <- n
+    n <- length(id.vect)
+    
+    if (is.null(sdvec) & geetype=="gaussian"){
+      sdvec <- rep(1,m)
+    }
+    
+    if (is.null(timedep_cor)){
+      timedep_cor <- 0.4
+    }
+    if (is.null(timedep_slope)){
+      timedep_slope <- 0.5
+    }else{
+      timedep_slope <- timedep_slope
+    }
+    
   }
   
   x <- xobs <- matrix(NA,nrow=n,ncol=p)
@@ -93,7 +125,7 @@ simu <- function(n = 100,
   
   # if (method == "manual"){
   
-  if (model != "timedep"){
+  if (!(model %in% c("timedep","gee"))){
     
     sigma <- rho^(as.matrix(dist(1:p)))
     diag(sigma) <- c(rep(log(p)/2,3),1,rep(log(p)/2,2),1,log(p)/2,rep(1,p-8))
@@ -108,7 +140,7 @@ simu <- function(n = 100,
       }
     }
     
-  }else if(model == "timedep"){
+  }else if(model %in% c("timedep","gee")){
     
     sigma <- rho^(as.matrix(dist(1:(p-(weak+strong)))))
     diag(sigma) <- 1
@@ -125,7 +157,7 @@ simu <- function(n = 100,
       # Mu <- t(mu + t(slopes))
       
       slopes <- rep(0:1,(weak+strong)/2)*timedep_slope
-      Mu <- t(c(rep(log(p),3),0,rep(log(p),2),0,log(p),rep(0,2)) + t(outer(0:9,slopes)))
+      Mu <- t(c(rep(log(p),3),0,rep(log(p),2),0,log(p),rep(0,2)) + t(outer(0:(m-1),slopes)))
       
       sigma1 <- rho^(as.matrix(dist(1:(weak+strong))))
       Sigma <- (diag(m) %x% sigma1) + ((matrix(1,nrow=m,ncol=m) - diag(m)) %x% (diag(strong+weak)*timedep_cor))
@@ -133,7 +165,7 @@ simu <- function(n = 100,
       # sigma_offdiag <- diag(strong+weak)*0.8
       # mat_template <- matrix(1,nrow=m,ncol=m) - diag(m)
       
-      x1 <- matrix(mvtnorm::rmvnorm(n=1,mean=as.vector(Mu),sigma=Sigma),nrow=m,ncol=weak+strong,byrow=FALSE)
+      x1 <- matrix(rmvnorm(n=1,mean=as.vector(Mu),sigma=Sigma),nrow=m,ncol=weak+strong,byrow=FALSE)
       x[id.vect==i,true_set] <- x1
       
       if (pct.sparsity > 0){
@@ -430,9 +462,9 @@ simu <- function(n = 100,
     
     colnames(data) <- c('id', 't', 't0', 'd', paste0("z",1:p))
     data <- data.frame(data)
-    data_unique <- data |> 
-      group_by(id) |> 
-      filter(row_number() == n()) |> 
+    data_unique <- data %>% 
+      group_by(id) %>% 
+      filter(row_number() == n()) %>% 
       ungroup()
     
     xcount <- data[,-c(1:4)]
@@ -450,6 +482,92 @@ simu <- function(n = 100,
                 beta=betavec,
                 idx=true_set)
     
+  }else if (model == "gee"){
+    
+    tvec <- rep(0:(m-1),n0)
+    
+    if (geetype == "gaussian"){
+      
+      if (corstr == "independence"){
+        
+        SIGMA <- diag(sdvec) %*% diag(m) %*% diag(sdvec)
+        
+      }else if (corstr == "exchangeable"){
+        
+        R <- matrix(rhogee,m,m)+diag(rep(1-rhogee,m)) # Working correlation matrix
+        SIGMA <- diag(sdvec) %*% R %*% diag(sdvec)
+        
+      }else if (corstr == "AR-1"){
+        
+        R <- matrix(NA,nrow=m,ncol=m)
+        for (t1 in 1:m) {
+          for (t2 in 1:m) {
+            R[t1,t2]<-rhogee^abs(t1-t2)   
+          }
+        }
+        SIGMA <- diag(sdvec) %*% R %*% diag(sdvec)
+        
+      }
+      
+      # covariance matrix of error
+      error <- rmvnorm(n0, mean = rep(0,m),SIGMA)
+      
+      # form continuous longitudinal outcomes
+      y <- tvec*geeslope + x[,true_set] %*% beta + as.vector(t(error))
+      
+      if(intercept) {
+        intcpt <- rnorm(1,mean=1,sd=1)
+        y <- y + intcpt
+      }
+      
+    }else if (geetype == "binomial"){
+      
+      eta <- tvec*geeslope + x[,true_set] %*% beta
+      if(intercept) {
+        intcpt <- rnorm(1,mean=1,sd=1)
+        eta <- eta + intcpt
+      }
+      prob <- exp(eta)/(1+exp(eta))
+      y <- rep(NA,n)
+      
+      if (corstr == "independence"){
+        
+        for (i in 1:n){
+          y[i] <- rbinom(1,1,prob=prob[i])
+        }
+        
+      }else if (corstr == "exchangeable"){
+        
+        for (i in 1:n0){
+          
+          mu <- prob[id.vect==i]
+          y[id.vect==i] <- binsimuexch(mu,rhogee)
+          
+        }
+        
+      }else if (corstr == "AR-1"){
+        
+        for (i in 1:n0){
+          
+          mu <- prob[id.vect==i]
+          y[id.vect==i] <- binsimuar1(mu,rhogee)
+          
+        }
+        
+      }
+      
+    }
+    
+    ret <- list(xcount=xcount,
+                x=xobs,
+                y=y,
+                id=id.vect,
+                tvec=tvec,
+                beta=betavec,
+                idx=true_set)
+    
+    if (intercept) ret$intercept=intcpt
+    
   }
   
   return(ret)
